//=============================================================================
// PowerHandler.
// 	Author: Pravin P. Prabhu
//	Revision: 1.1, 11/20/08
//	Abstract:
//		Handles the various powers that players may use. A player may forward
//	commands to the handler via the 'Mutate' command (this assumes xBaseCommands
//	is in the player's inventory). Players may forward a command with the syntax
//	"Mutate Power X" such that "X" is replaced with the power's callsign.
//	
//=============================================================================
class PowerHandler expands MutatorPlus
config(PlayerPowers);

// Loads
#EXEC OBJ LOAD FILE=..\SharedSystem\xCoop.u

// Constants
const CLASS_PLAYERPAWN = Class'PlayerPawn';
const CLASS_XBASECOMMANDS = Class'xBaseCommands';

const COMMAND_PREFIX = "POWER ";				// All mutate calls to Powers must be prefaced with this
const HELP_COMMAND = "HELP";				// Mutate COMMAND_PREFIX HELP_COMMAND probes a list of usable commands (powers)
const LOCALPLAYER_RECHECK_TIME = 0.5;

// Structs
struct Power_CFG
{
	var() config string powerclass;
	var() config string newxCoopPasswordOfUser;
};

// Configurable/external variables
var(Server) config Power_CFG UsablePower[256];		// Up to 256 unique powers may be used

// Class-wide variables
var Private Power PowerList[256];
Var Private Playerpawn LocalPlayer;	// Client variable used to delegate render calls..
var private int PowerListSize;
var private bool bAlreadyLoaded;	// WHY do mutators get postbeginplay twice??

//===========
// Functions 
//===========

// PostBeginPlay
//	Override to get local player
//
//	Abstract:
//		First, attempts to get the local player reference, which, if fails, forwards to Timer.
//	Second, this function loads the powers as dictated by the server owner's configuration.
//
Simulated Event Postbeginplay()
{
	if( bAlreadyLoaded )
		Return;
	bAlreadyLoaded = True;

	Super.postbeginplay();
	SaveConfig();	// Generate INI
	If( Level.NetMode != NM_DedicatedServer )
	{
		If( Level.netmode != nm_standalone ) 	// No need to check on standalone; always null
			Foreach AllActors(CLASS_PLAYERPAWN,LocalPlayer)
				If( LocalPlayer.Role==Role_Authority || LocalPlayer.Role == Role_AutonomousProxy )
					break;
		If( LocalPlayer == None )
		{
			// Fuck you.
			SetTimer(LOCALPLAYER_RECHECK_TIME,False);
		}
	}
	
	If( Level.NetMode != NM_Client )
		LoadPowers();
}


// Timer
//	Gets the local player if it failed in PostBeginPlas, loads powers
//
//	Abstract:
//		Checks every "localplayer_recheck_time" seconds to attain the localplayer reference. If the
//	localplayer is acquired, the timer stops. This is more of a half-assed fix for late replicating
//	actors online.
//
Simulated Event Timer()
{
	Foreach AllActors(CLASS_PLAYERPAWN,LocalPlayer)
		If( LocalPlayer.Role==Role_Authority || LocalPlayer.Role == Role_AutonomousProxy )
			break;
	If( LocalPlayer == None )
	{
		// Fuck you!!!!
		SetTimer(LOCALPLAYER_RECHECK_TIME,False);
	}
	Else if ( Level.netmode == nm_StandAlone ) 	// Standalone doesn't get mutateNewPlayer..
	{
		// Sw33t!
		MutateNewPlayer( LocalPlayer );
	}
}


// LoadPowers
//	Loads power classes
//	
//	Abstract:
//		Iterates through the array of powers that can be used and loads each into memory.
//
Function LoadPowers()
{
  local int index;			 // Next index to load from config
  local int load_index;	   // Next index to store into memory
  local class<power> temp;	 // Temp var for load checks
  local power tempPower;
  
  for( index = 0; index < arraycount(usablepower); index++ )
  {
  	If( UsablePower[Index].Powerclass != "" )
  	{
  		Temp = Class<Power>(DynamicLoadObject(Usablepower[index].powerclass,class'class'));
  		If( Temp == None )
  		{
  			// Bad variable
  			Log("Warning - Failed to load "$UsablePower[Index].PowerClass$"; this isn't a usable power. Please double check that this is correct.",'PowerHandler');
  			Continue;
  		}
  		
  		// Instantiate it
  		TempPower = Spawn(Temp);
  		If( TempPower == None )
  		{
  			Log("Warning - Failed to spawn"@TempPower@"...",'PowerHandler');
  			Continue;
  		}
  		TempPower.AllowedPassword = UsablePower[Index].NewxCoopPasswordOfUser;
  		If( TempPower.AllowedPassword == "" )
  		{
  			Log("Warning - "$TempPower@"at slot"@Index@"does NOT have a specific user. Currently being unused.",'PowerHandler');
  		}
  		
  		// Store it in the list
  		PowerList[load_index] = TempPower;
  		
  		load_index++;
  	}
  }
  
  // This index shall be used to traverse the array
  PowerListSize = load_Index;
}


// MutateClientLoadedPassword
//	Override to forward notification of specific users to Powers
//	
//	Abstract:
//		This is called when a client has just successfully replicated their newxCoopPassword to the server.
//	We will use this override to notify all Power objects as to their users entering the game.
//
function MutateClientLoadedPassword( XPRI ClientXPRI )
{
local int index; // For array traversal of power dynamic array
local string SenderPassword; // Password of incoming client

	SenderPassword = (ClientxPRI.newxCoopPassword);

	// Traverse the power dynamic array and find things this player can use
	for( index = 0; index < powerListSize; index++ )
	{
		If( Powerlist[index].AllowsUsage(Senderpassword) )
		{
			// This is a user of this command, notify it of its user entering!
			Powerlist[Index].UserEntering( Pawn(ClientxPRI.Owner) );
		}
	}
}


// MutateExitingPlayer
//	Override to notify Power actors of their users leaving the game
//	
//	Abstract:
//		This is called when a client has just successfully replicated their newxCoopPassword to the server.
//	We will use this override to notify all Power objects as to their users entering the game.
//
function MutateExitingPlayer( pawn Exiting )
{
local int index; // For array traversal of power dynamic array
local string SenderPassword; // Password of incoming client

	If( Exiting.IsA('PlayerPawn') &&
		XPRI(Exiting.PlayerReplicationInfo) != None )
	{
		SenderPassword = XPRI(Exiting.playerReplicationInfo).newxCoopPassword;
	
		// Traverse the power dynamic array and find things this player can use
		for( index = 0; index < powerListSize; index++ )
		{
			If( Powerlist[index].AllowsUsage(Senderpassword) )
			{
				// This is a user of this command, notify it of its user entering!
				Powerlist[Index].UserLeaving( Exiting );
			}
		}
	}
}


// Mutate
//	Override to forward commands server-side
//	
//	Abstract:
//		Takes in commands client-side and forwards them server-side IF proper syntax is used: Mutate Power ____
//
simulated function Mutate( Pawn Sender, Coerce string Data )
{
  local xBaseCommands XBC;

	// See if correct syntax was used: Mutate Power ___
	If( !(Left( Data, Len(COMMAND_PREFIX) ) ~= COMMAND_PREFIX) )
		return;
	
	Data = CAPS(Data);
	XBC = xBaseCommands(Sender.FindInventoryType(CLASS_XBASECOMMANDS));
	If( XBC == None )
		Return;	
		
	// Forward server-side for execution
	XBC.ServerMutate( Data );
}


// ServerMutate
//	Override to analyze incoming commands
//
//	Abstract:
//		Analyzes command for proper syntax. Parses out command. Forwards command to "Power"
//	actors. If the command is found to be "help", then a list of usable commands (to the user)
//	is outputted.
//
Function ServerMutate( Pawn Sender, Coerce String Data, optional MutatorPlus ForwardTO )
{
  local string Command;  // Parsed out command from the mutate call
  local int index;		   // For traversing the power array
  local string senderPassword;	// Password of the user
  local string Extra;		// Output buffer
  
	// See if correct syntax was used: Mutate Power ___
	If( !(Left( Data, Len(COMMAND_PREFIX) ) ~= COMMAND_PREFIX) )
		return;
	
	// Parse out the command
	Command = Right( Data, len(Data) - len(COMMAND_PREFIX) );
	Command = Caps(Command);
		
	// Load pass for verification
	SenderPassword = (XPRI(Sender.PlayerReplicationInfo).newxCoopPassword);
	
	If( Command ~= HELP_COMMAND )
	{
		If( Sender != None )
		{
			Extra = "";	// Clear buffer to prepare for output
			Sender.ClientMessage("The following is a list of powers you may use:",'ServerEvent',True);
			Sender.ClientMessage("Example usage: Mutate Power MyPower",'ServerEvent',True);
			for( index = 0; index < powerListSize; index++ )
			{
				If( !Powerlist[index].AllowsUsage(Senderpassword) )
					Continue;	// This command is not able to be executed by this player!
			
				// Attempt to add to buffer
				If( Len( Extra ) + Len( PowerList[Index].PowerName() ) + 2 > 255 )	 // The 2 is for the necessary ", " comma and space for appending stuff
				{
					// Failed to add to buffer; flush and add to empty buffer
					Sender.ClientMessage(Extra);
					Extra = PowerList[Index].PowerName();
				}
				Else
				{
					// Valid - perform add
					If( Extra == "" )
						Extra = PowerList[Index].PowerName();
					Else
						Extra = Extra$", "$PowerList[Index].PowerName();
				}
			}
			
			If( Extra != "" )
				Sender.ClientMessage(Extra);	// Flush the buffer
			Else
				Sender.ClientMessage("There are no usable powers.");
		}
		return;
	}


	// Find move set
	For( index = 0; index < powerListSize; index++ )
	{
		If( PowerList[Index].PowerName() ~= Command )
		{
			// This is the right power; for offline games, allow usage of anything for testing
			If( Level.NetMode == NM_StandAlone || PowerList[Index].AllowsUsage( SenderPassword ) )
				PowerList[Index].UseFor( Sender );
		}
	}
}


// MutatorScoreKill
//	Override to forward Kill and Died signals to Powers, if a User dies or gets
//	a kill
//
//	Abstract:
//		Function is called when a pawn kills another pawn, or a pawn dies. If one of
//	these is a user of one of the powers, then this triggers a call to UserScoredKill
//	or UserDied in their power.
//
function MutatorScoreKill( Pawn Killer, Pawn Other )
{
local int index; 	// Index which drives array traversal when looking up powers that
					// the killer/dying pawn is a user of
					
local string password;	// Loaded password when analyzing scenarios, prior
						// to forwarding to the powers
						
	If( Killer == Other )
	{
		Killer = None;	// Treat it as if the pawn just died
	}
	
	
	// Forward the scoring of a point by a user to their powers
	If( Killer != None &&									// If there was a killer,
		Killer.IsA('PlayerPawn') &&							// and it's a player
		Killer.PlayerReplicationInfo != None && 			// and it has a PRI
		XPRI( Killer.PlayerReplicationInfo ) != None )		// an XPRI to be exact...
		{
			password = XPRI(Killer.Playerreplicationinfo).newxCoopPassword;
			for( index = 0; index < powerListSize; index++ )
			{
				If( Powerlist[index].AllowsUsage(password) )
				{
					// Killer is a user of this Power, so forward the UserKilledPawn
					// call to let the Power know about this
					PowerList[index].UserScoredKill( Killer, Other );
				}
			}
		}
	
	
	// Forward death of a user to their powers
	If( Other != None &&
		Other.IsA('PlayerPawn') &&
		Other.PlayerReplicationInfo != None &&
		XPRI( Other.PlayerReplicationInfo ) != none )
		{
			password = XPRI(Other.Playerreplicationinfo).newxCoopPassword;
			for( index = 0; index < powerListSize; index++ )
			{
				If( Powerlist[index].AllowsUsage(password) )
				{
					// Killer is a user of this Power, so forward the UserKilledPawn
					// call to let the Power know about this
					PowerList[index].UserDied( Other );
				}
			}
		}
}


// MutatorReduceDamage
//	Override to forward Take and Deal damage calls of users
//
//	Abstract:
//		Called when damage is being passed around.
//
function MutatorReduceDamage( out int Damage, name Type, pawn Injured, pawn InstigatedBy )
{
local int index; 	// Index which drives array traversal when looking up powers that
					// the killer/dying pawn is a user of
					
local string password;	// Loaded password when analyzing scenarios, prior
						// to forwarding to the powers

	
	// Forward the scoring of a point by a user to their powers
	If( injured != None &&									// If there was a killer,
		injured.IsA('PlayerPawn') &&							// and it's a player
		injured.PlayerReplicationInfo != None && 			// and it has a PRI
		XPRI( injured.PlayerReplicationInfo ) != None )		// an XPRI to be exact...
		{
			password = XPRI(injured.Playerreplicationinfo).newxCoopPassword;
			for( index = 0; index < powerListSize; index++ )
			{
				If( Powerlist[index].AllowsUsage(password) )
				{
					// Killer is a user of this Power, so forward the UserKilledPawn
					// call to let the Power know about this
					PowerList[index].UserTakingDamage( Damage, Type, InstigatedBy );
				}
			}
		}
	
	
	// Forward death of a user to their powers
	If( instigatedBy != None &&
		instigatedBy.IsA('PlayerPawn') &&
		instigatedBy.PlayerReplicationInfo != None &&
		XPRI( instigatedBy.PlayerReplicationInfo ) != none )
		{
			password = XPRI(instigatedBy.Playerreplicationinfo).newxCoopPassword;
			for( index = 0; index < powerListSize; index++ )
			{
				If( Powerlist[index].AllowsUsage(password) )
				{
					// Killer is a user of this Power, so forward the UserKilledPawn
					// call to let the Power know about this
					PowerList[index].UserDealingDamage( Damage, Type, Injured );
				}
			}
		}
}


// LevelEnd
//	Override to forward Cleanup signal to Powers
//
//	Abstract:
//		Called when the level is ending. Forward cleanup call to the Power actors.
//
Function LevelEnd()
{
local int index; // For array traversal through power list

	// Find move set
	For( index = 0; index < powerListSize; index++ )
		PowerList[Index].Cleanup();
}

defaultproperties
{
				UsablePower(0)=(powerclass="PWR_Fireball.Fireball",newxCoopPasswordOfUser="StandAlone")
				UsablePower(1)=(newxCoopPasswordOfUser="0")
				UsablePower(2)=(newxCoopPasswordOfUser="0")
				UsablePower(3)=(newxCoopPasswordOfUser="0")
				UsablePower(4)=(newxCoopPasswordOfUser="0")
				UsablePower(5)=(newxCoopPasswordOfUser="0")
				UsablePower(6)=(newxCoopPasswordOfUser="0")
				UsablePower(7)=(newxCoopPasswordOfUser="0")
				UsablePower(8)=(newxCoopPasswordOfUser="0")
				UsablePower(9)=(newxCoopPasswordOfUser="0")
				UsablePower(10)=(newxCoopPasswordOfUser="0")
				UsablePower(11)=(newxCoopPasswordOfUser="0")
				UsablePower(12)=(newxCoopPasswordOfUser="0")
				UsablePower(13)=(newxCoopPasswordOfUser="0")
				UsablePower(14)=(newxCoopPasswordOfUser="0")
				UsablePower(15)=(newxCoopPasswordOfUser="0")
				UsablePower(16)=(newxCoopPasswordOfUser="0")
				UsablePower(17)=(newxCoopPasswordOfUser="0")
				UsablePower(18)=(newxCoopPasswordOfUser="0")
				UsablePower(19)=(newxCoopPasswordOfUser="0")
				UsablePower(20)=(newxCoopPasswordOfUser="0")
				UsablePower(21)=(newxCoopPasswordOfUser="0")
				UsablePower(22)=(newxCoopPasswordOfUser="0")
				UsablePower(23)=(newxCoopPasswordOfUser="0")
				UsablePower(24)=(newxCoopPasswordOfUser="0")
				UsablePower(25)=(newxCoopPasswordOfUser="0")
				UsablePower(26)=(newxCoopPasswordOfUser="0")
				UsablePower(27)=(newxCoopPasswordOfUser="0")
				UsablePower(28)=(newxCoopPasswordOfUser="0")
				UsablePower(29)=(newxCoopPasswordOfUser="0")
				UsablePower(30)=(newxCoopPasswordOfUser="0")
				UsablePower(31)=(newxCoopPasswordOfUser="0")
				UsablePower(32)=(newxCoopPasswordOfUser="0")
				UsablePower(33)=(newxCoopPasswordOfUser="0")
				UsablePower(34)=(newxCoopPasswordOfUser="0")
				UsablePower(35)=(newxCoopPasswordOfUser="0")
				UsablePower(36)=(newxCoopPasswordOfUser="0")
				UsablePower(37)=(newxCoopPasswordOfUser="0")
				UsablePower(38)=(newxCoopPasswordOfUser="0")
				UsablePower(39)=(newxCoopPasswordOfUser="0")
				UsablePower(40)=(newxCoopPasswordOfUser="0")
				UsablePower(41)=(newxCoopPasswordOfUser="0")
				UsablePower(42)=(newxCoopPasswordOfUser="0")
				UsablePower(43)=(newxCoopPasswordOfUser="0")
				UsablePower(44)=(newxCoopPasswordOfUser="0")
				UsablePower(45)=(newxCoopPasswordOfUser="0")
				UsablePower(46)=(newxCoopPasswordOfUser="0")
				UsablePower(47)=(newxCoopPasswordOfUser="0")
				UsablePower(48)=(newxCoopPasswordOfUser="0")
				UsablePower(49)=(newxCoopPasswordOfUser="0")
				UsablePower(50)=(newxCoopPasswordOfUser="0")
				UsablePower(51)=(newxCoopPasswordOfUser="0")
				UsablePower(52)=(newxCoopPasswordOfUser="0")
				UsablePower(53)=(newxCoopPasswordOfUser="0")
				UsablePower(54)=(newxCoopPasswordOfUser="0")
				UsablePower(55)=(newxCoopPasswordOfUser="0")
				UsablePower(56)=(newxCoopPasswordOfUser="0")
				UsablePower(57)=(newxCoopPasswordOfUser="0")
				UsablePower(58)=(newxCoopPasswordOfUser="0")
				UsablePower(59)=(newxCoopPasswordOfUser="0")
				UsablePower(60)=(newxCoopPasswordOfUser="0")
				UsablePower(61)=(newxCoopPasswordOfUser="0")
				UsablePower(62)=(newxCoopPasswordOfUser="0")
				UsablePower(63)=(newxCoopPasswordOfUser="0")
				UsablePower(64)=(newxCoopPasswordOfUser="0")
				UsablePower(65)=(newxCoopPasswordOfUser="0")
				UsablePower(66)=(newxCoopPasswordOfUser="0")
				UsablePower(67)=(newxCoopPasswordOfUser="0")
				UsablePower(68)=(newxCoopPasswordOfUser="0")
				UsablePower(69)=(newxCoopPasswordOfUser="0")
				UsablePower(70)=(newxCoopPasswordOfUser="0")
				UsablePower(71)=(newxCoopPasswordOfUser="0")
				UsablePower(72)=(newxCoopPasswordOfUser="0")
				UsablePower(73)=(newxCoopPasswordOfUser="0")
				UsablePower(74)=(newxCoopPasswordOfUser="0")
				UsablePower(75)=(newxCoopPasswordOfUser="0")
				UsablePower(76)=(newxCoopPasswordOfUser="0")
				UsablePower(77)=(newxCoopPasswordOfUser="0")
				UsablePower(78)=(newxCoopPasswordOfUser="0")
				UsablePower(79)=(newxCoopPasswordOfUser="0")
				UsablePower(80)=(newxCoopPasswordOfUser="0")
				UsablePower(81)=(newxCoopPasswordOfUser="0")
				UsablePower(82)=(newxCoopPasswordOfUser="0")
				UsablePower(83)=(newxCoopPasswordOfUser="0")
				UsablePower(84)=(newxCoopPasswordOfUser="0")
				UsablePower(85)=(newxCoopPasswordOfUser="0")
				UsablePower(86)=(newxCoopPasswordOfUser="0")
				UsablePower(87)=(newxCoopPasswordOfUser="0")
				UsablePower(88)=(newxCoopPasswordOfUser="0")
				UsablePower(89)=(newxCoopPasswordOfUser="0")
				UsablePower(90)=(newxCoopPasswordOfUser="0")
				UsablePower(91)=(newxCoopPasswordOfUser="0")
				UsablePower(92)=(newxCoopPasswordOfUser="0")
				UsablePower(93)=(newxCoopPasswordOfUser="0")
				UsablePower(94)=(newxCoopPasswordOfUser="0")
				UsablePower(95)=(newxCoopPasswordOfUser="0")
				UsablePower(96)=(newxCoopPasswordOfUser="0")
				UsablePower(97)=(newxCoopPasswordOfUser="0")
				UsablePower(98)=(newxCoopPasswordOfUser="0")
				UsablePower(99)=(newxCoopPasswordOfUser="0")
				UsablePower(100)=(newxCoopPasswordOfUser="0")
				UsablePower(101)=(newxCoopPasswordOfUser="0")
				UsablePower(102)=(newxCoopPasswordOfUser="0")
				UsablePower(103)=(newxCoopPasswordOfUser="0")
				UsablePower(104)=(newxCoopPasswordOfUser="0")
				UsablePower(105)=(newxCoopPasswordOfUser="0")
				UsablePower(106)=(newxCoopPasswordOfUser="0")
				UsablePower(107)=(newxCoopPasswordOfUser="0")
				UsablePower(108)=(newxCoopPasswordOfUser="0")
				UsablePower(109)=(newxCoopPasswordOfUser="0")
				UsablePower(110)=(newxCoopPasswordOfUser="0")
				UsablePower(111)=(newxCoopPasswordOfUser="0")
				UsablePower(112)=(newxCoopPasswordOfUser="0")
				UsablePower(113)=(newxCoopPasswordOfUser="0")
				UsablePower(114)=(newxCoopPasswordOfUser="0")
				UsablePower(115)=(newxCoopPasswordOfUser="0")
				UsablePower(116)=(newxCoopPasswordOfUser="0")
				UsablePower(117)=(newxCoopPasswordOfUser="0")
				UsablePower(118)=(newxCoopPasswordOfUser="0")
				UsablePower(119)=(newxCoopPasswordOfUser="0")
				UsablePower(120)=(newxCoopPasswordOfUser="0")
				UsablePower(121)=(newxCoopPasswordOfUser="0")
				UsablePower(122)=(newxCoopPasswordOfUser="0")
				UsablePower(123)=(newxCoopPasswordOfUser="0")
				UsablePower(124)=(newxCoopPasswordOfUser="0")
				UsablePower(125)=(newxCoopPasswordOfUser="0")
				UsablePower(126)=(newxCoopPasswordOfUser="0")
				UsablePower(127)=(newxCoopPasswordOfUser="0")
				UsablePower(128)=(newxCoopPasswordOfUser="0")
				UsablePower(129)=(newxCoopPasswordOfUser="0")
				UsablePower(130)=(newxCoopPasswordOfUser="0")
				UsablePower(131)=(newxCoopPasswordOfUser="0")
				UsablePower(132)=(newxCoopPasswordOfUser="0")
				UsablePower(133)=(newxCoopPasswordOfUser="0")
				UsablePower(134)=(newxCoopPasswordOfUser="0")
				UsablePower(135)=(newxCoopPasswordOfUser="0")
				UsablePower(136)=(newxCoopPasswordOfUser="0")
				UsablePower(137)=(newxCoopPasswordOfUser="0")
				UsablePower(138)=(newxCoopPasswordOfUser="0")
				UsablePower(139)=(newxCoopPasswordOfUser="0")
				UsablePower(140)=(newxCoopPasswordOfUser="0")
				UsablePower(141)=(newxCoopPasswordOfUser="0")
				UsablePower(142)=(newxCoopPasswordOfUser="0")
				UsablePower(143)=(newxCoopPasswordOfUser="0")
				UsablePower(144)=(newxCoopPasswordOfUser="0")
				UsablePower(145)=(newxCoopPasswordOfUser="0")
				UsablePower(146)=(newxCoopPasswordOfUser="0")
				UsablePower(147)=(newxCoopPasswordOfUser="0")
				UsablePower(148)=(newxCoopPasswordOfUser="0")
				UsablePower(149)=(newxCoopPasswordOfUser="0")
				UsablePower(150)=(newxCoopPasswordOfUser="0")
				UsablePower(151)=(newxCoopPasswordOfUser="0")
				UsablePower(152)=(newxCoopPasswordOfUser="0")
				UsablePower(153)=(newxCoopPasswordOfUser="0")
				UsablePower(154)=(newxCoopPasswordOfUser="0")
				UsablePower(155)=(newxCoopPasswordOfUser="0")
				UsablePower(156)=(newxCoopPasswordOfUser="0")
				UsablePower(157)=(newxCoopPasswordOfUser="0")
				UsablePower(158)=(newxCoopPasswordOfUser="0")
				UsablePower(159)=(newxCoopPasswordOfUser="0")
				UsablePower(160)=(newxCoopPasswordOfUser="0")
				UsablePower(161)=(newxCoopPasswordOfUser="0")
				UsablePower(162)=(newxCoopPasswordOfUser="0")
				UsablePower(163)=(newxCoopPasswordOfUser="0")
				UsablePower(164)=(newxCoopPasswordOfUser="0")
				UsablePower(165)=(newxCoopPasswordOfUser="0")
				UsablePower(166)=(newxCoopPasswordOfUser="0")
				UsablePower(167)=(newxCoopPasswordOfUser="0")
				UsablePower(168)=(newxCoopPasswordOfUser="0")
				UsablePower(169)=(newxCoopPasswordOfUser="0")
				UsablePower(170)=(newxCoopPasswordOfUser="0")
				UsablePower(171)=(newxCoopPasswordOfUser="0")
				UsablePower(172)=(newxCoopPasswordOfUser="0")
				UsablePower(173)=(newxCoopPasswordOfUser="0")
				UsablePower(174)=(newxCoopPasswordOfUser="0")
				UsablePower(175)=(newxCoopPasswordOfUser="0")
				UsablePower(176)=(newxCoopPasswordOfUser="0")
				UsablePower(177)=(newxCoopPasswordOfUser="0")
				UsablePower(178)=(newxCoopPasswordOfUser="0")
				UsablePower(179)=(newxCoopPasswordOfUser="0")
				UsablePower(180)=(newxCoopPasswordOfUser="0")
				UsablePower(181)=(newxCoopPasswordOfUser="0")
				UsablePower(182)=(newxCoopPasswordOfUser="0")
				UsablePower(183)=(newxCoopPasswordOfUser="0")
				UsablePower(184)=(newxCoopPasswordOfUser="0")
				UsablePower(185)=(newxCoopPasswordOfUser="0")
				UsablePower(186)=(newxCoopPasswordOfUser="0")
				UsablePower(187)=(newxCoopPasswordOfUser="0")
				UsablePower(188)=(newxCoopPasswordOfUser="0")
				UsablePower(189)=(newxCoopPasswordOfUser="0")
				UsablePower(190)=(newxCoopPasswordOfUser="0")
				UsablePower(191)=(newxCoopPasswordOfUser="0")
				UsablePower(192)=(newxCoopPasswordOfUser="0")
				UsablePower(193)=(newxCoopPasswordOfUser="0")
				UsablePower(194)=(newxCoopPasswordOfUser="0")
				UsablePower(195)=(newxCoopPasswordOfUser="0")
				UsablePower(196)=(newxCoopPasswordOfUser="0")
				UsablePower(197)=(newxCoopPasswordOfUser="0")
				UsablePower(198)=(newxCoopPasswordOfUser="0")
				UsablePower(199)=(newxCoopPasswordOfUser="0")
				UsablePower(200)=(newxCoopPasswordOfUser="0")
				UsablePower(201)=(newxCoopPasswordOfUser="0")
				UsablePower(202)=(newxCoopPasswordOfUser="0")
				UsablePower(203)=(newxCoopPasswordOfUser="0")
				UsablePower(204)=(newxCoopPasswordOfUser="0")
				UsablePower(205)=(newxCoopPasswordOfUser="0")
				UsablePower(206)=(newxCoopPasswordOfUser="0")
				UsablePower(207)=(newxCoopPasswordOfUser="0")
				UsablePower(208)=(newxCoopPasswordOfUser="0")
				UsablePower(209)=(newxCoopPasswordOfUser="0")
				UsablePower(210)=(newxCoopPasswordOfUser="0")
				UsablePower(211)=(newxCoopPasswordOfUser="0")
				UsablePower(212)=(newxCoopPasswordOfUser="0")
				UsablePower(213)=(newxCoopPasswordOfUser="0")
				UsablePower(214)=(newxCoopPasswordOfUser="0")
				UsablePower(215)=(newxCoopPasswordOfUser="0")
				UsablePower(216)=(newxCoopPasswordOfUser="0")
				UsablePower(217)=(newxCoopPasswordOfUser="0")
				UsablePower(218)=(newxCoopPasswordOfUser="0")
				UsablePower(219)=(newxCoopPasswordOfUser="0")
				UsablePower(220)=(newxCoopPasswordOfUser="0")
				UsablePower(221)=(newxCoopPasswordOfUser="0")
				UsablePower(222)=(newxCoopPasswordOfUser="0")
				UsablePower(223)=(newxCoopPasswordOfUser="0")
				UsablePower(224)=(newxCoopPasswordOfUser="0")
				UsablePower(225)=(newxCoopPasswordOfUser="0")
				UsablePower(226)=(newxCoopPasswordOfUser="0")
				UsablePower(227)=(newxCoopPasswordOfUser="0")
				UsablePower(228)=(newxCoopPasswordOfUser="0")
				UsablePower(229)=(newxCoopPasswordOfUser="0")
				UsablePower(230)=(newxCoopPasswordOfUser="0")
				UsablePower(231)=(newxCoopPasswordOfUser="0")
				UsablePower(232)=(newxCoopPasswordOfUser="0")
				UsablePower(233)=(newxCoopPasswordOfUser="0")
				UsablePower(234)=(newxCoopPasswordOfUser="0")
				UsablePower(235)=(newxCoopPasswordOfUser="0")
				UsablePower(236)=(newxCoopPasswordOfUser="0")
				UsablePower(237)=(newxCoopPasswordOfUser="0")
				UsablePower(238)=(newxCoopPasswordOfUser="0")
				UsablePower(239)=(newxCoopPasswordOfUser="0")
				UsablePower(240)=(newxCoopPasswordOfUser="0")
				UsablePower(241)=(newxCoopPasswordOfUser="0")
				UsablePower(242)=(newxCoopPasswordOfUser="0")
				UsablePower(243)=(newxCoopPasswordOfUser="0")
				UsablePower(244)=(newxCoopPasswordOfUser="0")
				UsablePower(245)=(newxCoopPasswordOfUser="0")
				UsablePower(246)=(newxCoopPasswordOfUser="0")
				UsablePower(247)=(newxCoopPasswordOfUser="0")
				UsablePower(248)=(newxCoopPasswordOfUser="0")
				UsablePower(249)=(newxCoopPasswordOfUser="0")
				UsablePower(250)=(newxCoopPasswordOfUser="0")
				UsablePower(251)=(newxCoopPasswordOfUser="0")
				UsablePower(252)=(newxCoopPasswordOfUser="0")
				UsablePower(253)=(newxCoopPasswordOfUser="0")
				UsablePower(254)=(newxCoopPasswordOfUser="0")
				UsablePower(255)=(newxCoopPasswordOfUser="0")
}
